@@ -16,6 +16,7 @@
 import argparse
 import logging
 import os
+import Queue
 import sys
 import threading
 import webbrowser
@@ -129,17 +130,30 @@ class SearchDialog(mywid.ButtonDialog):
 
 class App(object):
     def __init__(self, server=None, palette='default', keymap='default',
-                 debug=False, disable_sync=False, fetch_missing_refs=False,
-                 path=config.DEFAULT_CONFIG_PATH):
+                 debug=False, verbose=False, disable_sync=False,
+                 fetch_missing_refs=False, path=config.DEFAULT_CONFIG_PATH):
         self.server = server
         self.config = config.Config(server, palette, keymap, path)
         if debug:
             level = logging.DEBUG
+        elif verbose:
+            level = logging.INFO
         else:
             level = logging.WARNING
         logging.basicConfig(filename=self.config.log_file, filemode='w',
                             format='%(asctime)s %(message)s',
                             level=level)
+        # Python2.6 Logger.setLevel doesn't convert string name
+        # to integer code. Here, we set the requests logger level to
+        # be less verbose, since our logging output duplicates some
+        # requests logging content in places.
+        req_level_name = 'WARN'
+        req_logger = logging.getLogger('requests')
+        if sys.version_info < (2, 7):
+            level = logging.getLevelName(req_level_name)
+            req_logger.setLevel(level)
+        else:
+            req_logger.setLevel(req_level_name)
         self.log = logging.getLogger('gertty.App')
         self.log.debug("Starting")
 
@@ -177,7 +191,10 @@ class App(object):
             self.status.update(offline=True)
 
     def run(self):
-        self.loop.run()
+        try:
+            self.loop.run()
+        except KeyboardInterrupt:
+            pass
 
     def _quit(self, widget=None):
         raise urwid.ExitMainLoop()
@@ -208,7 +225,7 @@ class App(object):
         if hasattr(widget, 'title'):
             self.status.update(title=widget.title)
         self.loop.widget = widget
-        self.refresh()
+        self.refresh(force=True)
 
     def findChangeList(self):
         for widget in reversed(self.screens):
@@ -222,12 +239,21 @@ class App(object):
             widget = self.screens.pop()
             self.loop.widget = widget
 
-    def refresh(self, data=None):
+    def refresh(self, data=None, force=False):
         self.status.refresh()
         widget = self.loop.widget
         while isinstance(widget, urwid.Overlay):
             widget = widget.contents[0][0]
-        widget.refresh()
+        interested = force
+        try:
+            while True:
+                event = self.sync.result_queue.get(0)
+                if widget.interested(event):
+                    interested = True
+        except Queue.Empty:
+            pass
+        if interested:
+            widget.refresh()
 
     def popup(self, widget,
               relative_width=50, relative_height=25,
@@ -409,6 +435,8 @@ def main():
     parser.add_argument('-c', dest='path',
                         default=config.DEFAULT_CONFIG_PATH,
                         help='path to config file')
+    parser.add_argument('-v', dest='verbose', action='store_true',
+                        help='enable more verbose logging')
     parser.add_argument('-d', dest='debug', action='store_true',
                         help='enable debug logging')
     parser.add_argument('--no-sync', dest='no_sync', action='store_true',
@@ -422,16 +450,16 @@ def main():
                         help='print the palette attribute names to stdout')
     parser.add_argument('--version', dest='version', action='version',
                         version=version(),
-                        help='show zuul version')
+                        help='show Gertty\'s version')
     parser.add_argument('-p', dest='palette', default='default',
-                        help='Color palette to use')
+                        help='color palette to use')
     parser.add_argument('-k', dest='keymap', default='default',
-                        help='Keymap to use')
+                        help='keymap to use')
     parser.add_argument('server', nargs='?',
                         help='the server to use (as specified in config file)')
     args = parser.parse_args()
-    g = App(args.server, args.palette, args.keymap, args.debug, args.no_sync,
-            args.fetch_missing_refs, args.path)
+    g = App(args.server, args.palette, args.keymap, args.debug, args.verbose,
+            args.no_sync, args.fetch_missing_refs, args.path)
     g.run()
 
 
