@@ -83,12 +83,18 @@ class CommitContext(object):
             commit.authored_date, commit.author_tz_offset)
         commit_date = self.decorateGitTime(
             commit.committed_date, commit.committer_tz_offset)
+        if type(author.email) is unicode:
+            author_email = author.email
+        else:
+            author_email = unicode(author.email, 'utf8')
+        if type(committer.email) is unicode:
+            committer_email = committer.email
+        else:
+            committer_email = unicode(committer.email, 'utf8')
         return [u"Parent: %s\n" % parentsha,
-                u"Author: %s <%s>\n" % (author.name,
-                                        unicode(author.email, 'utf8')),
+                u"Author: %s <%s>\n" % (author.name, author_email),
                 u"AuthorDate: %s\n" % author_date,
-                u"Commit: %s <%s>\n" % (committer.name,
-                                        unicode(committer.email, 'utf')),
+                u"Commit: %s <%s>\n" % (committer.name, committer_email),
                 u"CommitDate: %s\n" % commit_date,
                 u"\n"] + commit.message.splitlines(True)
 
@@ -106,6 +112,8 @@ class CommitContext(object):
         self.deleted_file = False
         self.a_blob = CommitBlob()
         self.b_blob = CommitBlob()
+        self.a_path = self.a_blob.path
+        self.b_path = self.b_blob.path
         self.diff = ''.join(difflib.unified_diff(
             self.decorateMessage(old), self.decorateMessage(new),
             fromfile="/a/COMMIT_MSG", tofile="/b/COMMIT_MSG"))
@@ -160,8 +168,8 @@ class DiffChangedChunk(DiffChunk):
 
 class DiffFile(object):
     def __init__(self):
-        self.newname = None
-        self.oldname = None
+        self.newname = 'Unknown File'
+        self.oldname = 'Unknown File'
         self.chunks = []
         self.current_chunk = None
         self.old_lineno = 0
@@ -371,6 +379,10 @@ class Repo(object):
                 f.oldname = diff_context.a_blob.path
             if diff_context.b_blob:
                 f.newname = diff_context.b_blob.path
+            # TODO(jeblair): if/when https://github.com/gitpython-developers/GitPython/pull/266 merges,
+            # remove above 4 lines and replace with these two:
+            # f.oldname = diff_context.a_path
+            # f.newname = diff_context.b_path
             if diff_context.new_file:
                 f.oldname = 'Empty file'
             if diff_context.deleted_file:
@@ -448,6 +460,8 @@ class Repo(object):
                     continue
                 if line.startswith("index"):
                     continue
+                if line.startswith("Binary files"):
+                    continue
                 if not last_line:
                     raise Exception("Unhandled line: %s" % line)
             f.finalize()
