@@ -23,6 +23,10 @@ DEFAULT_PALETTE={
     'negative-label': ['dark red', ''],
     'max-label': ['light green', ''],
     'min-label': ['light red', ''],
+    'focused-positive-label': ['dark green,standout', ''],
+    'focused-negative-label': ['dark red,standout', ''],
+    'focused-max-label': ['light green,standout', ''],
+    'focused-min-label': ['light red,standout', ''],
     'link': ['dark blue', ''],
     'focused-link': ['light blue', ''],
     # Diff
@@ -44,7 +48,8 @@ DEFAULT_PALETTE={
     'line-number': ['dark gray', ''],
     'focused-line-number': ['dark gray,standout', ''],
     # Change view
-    'change-data': ['light cyan', ''],
+    'change-data': ['dark cyan', ''],
+    'focused-change-data': ['light cyan', ''],
     'change-header': ['light blue', ''],
     'revision-name': ['light blue', ''],
     'revision-commit': ['dark blue', ''],
@@ -86,6 +91,7 @@ LIGHT_PALETTE = {
     'focused-subscribed-project': ['dark gray,standout', ''],
     'focused-unsubscribed-project': ['dark gray,standout', ''],
     'change-data': ['dark blue,bold', ''],
+    'focused-change-data': ['dark blue,standout', ''],
     'reviewer-name': ['brown', ''],
     'change-message-name': ['brown', ''],
     'change-message-header': ['black', ''],
