@@ -78,6 +78,50 @@ class MultiQueue(object):
         finally:
             self.condition.release()
 
+class UpdateEvent(object):
+    def updateRelatedChanges(self, session, change):
+        related_change_keys = set()
+        related_change_keys.add(change.key)
+        for revision in change.revisions:
+            parent = session.getRevisionByCommit(revision.parent)
+            if parent:
+                related_change_keys.add(parent.change.key)
+            for child in session.getRevisionsByParent(revision.commit):
+                related_change_keys.add(child.change.key)
+        self.related_change_keys = related_change_keys
+
+class ProjectAddedEvent(UpdateEvent):
+    def __repr__(self):
+        return '<ProjectAddedEvent project_key:%s>' % (
+            self.project_key,)
+
+    def __init__(self, project):
+        self.project_key = project.key
+
+class ChangeAddedEvent(UpdateEvent):
+    def __repr__(self):
+        return '<ChangeAddedEvent project_key:%s change_key:%s>' % (
+            self.project_key, self.change_key)
+
+    def __init__(self, change):
+        self.project_key = change.project.key
+        self.change_key = change.key
+        self.related_change_keys = set()
+        self.review_flag_changed = True
+        self.status_changed = True
+
+class ChangeUpdatedEvent(UpdateEvent):
+    def __repr__(self):
+        return '<ChangeUpdatedEvent project_key:%s change_key:%s review_flag_changed:%s status_changed:%s>' % (
+            self.project_key, self.change_key, self.review_flag_changed, self.status_changed)
+
+    def __init__(self, change):
+        self.project_key = change.project.key
+        self.change_key = change.key
+        self.related_change_keys = set()
+        self.review_flag_changed = False
+        self.status_changed = False
+
 class Task(object):
     def __init__(self, priority=NORMAL_PRIORITY):
         self.log = logging.getLogger('gertty.sync')
@@ -85,6 +129,7 @@ class Task(object):
         self.succeeded = None
         self.event = threading.Event()
         self.tasks = []
+        self.results = []
 
     def complete(self, success):
         self.succeeded = success
@@ -127,7 +172,10 @@ class SyncProjectListTask(Task):
 
             for name in remote_keys-local_keys:
                 p = remote[name]
-                session.createProject(name, description=p.get('description', ''))
+                project = session.createProject(name,
+                                                description=p.get('description', ''))
+                self.log.info("Created project %s", project.name)
+                self.results.append(ProjectAddedEvent(project))
 
 class SyncSubscribedProjectBranchesTask(Task):
     def __repr__(self):
@@ -136,8 +184,9 @@ class SyncSubscribedProjectBranchesTask(Task):
     def run(self, sync):
         app = sync.app
         with app.db.getSession() as session:
-            for p in session.getProjects(subscribed=True):
-                sync.submitTask(SyncProjectBranchesTask(p.name, self.priority))
+            projects = session.getProjects(subscribed=True)
+        for p in projects:
+            sync.submitTask(SyncProjectBranchesTask(p.name, self.priority))
 
 class SyncProjectBranchesTask(Task):
     branch_re = re.compile(r'refs/heads/(.*)')
@@ -165,12 +214,12 @@ class SyncProjectBranchesTask(Task):
             local_branches = set(local.keys())
 
             for name in local_branches-remote_branches:
-                self.log.debug("Delete branch %s from project %s" % (name, project.name))
                 session.delete(local[name])
+                self.log.info("Deleted branch %s from project %s in local DB.", name, project.name)
 
             for name in remote_branches-local_branches:
-                self.log.debug("Add branch %s to project %s" % (name, project.name))
                 project.createBranch(name)
+                self.log.info("Added branch %s to project %s in local DB.", name, project.name)
 
 class SyncSubscribedProjectsTask(Task):
     def __repr__(self):
@@ -180,10 +229,10 @@ class SyncSubscribedProjectsTask(Task):
         app = sync.app
         with app.db.getSession() as session:
             keys = [p.key for p in session.getProjects(subscribed=True)]
-            for i in range(0, len(keys), 10):
-                t = SyncProjectTask(keys[i:i+10], self.priority)
-                self.tasks.append(t)
-                sync.submitTask(t)
+        for i in range(0, len(keys), 10):
+            t = SyncProjectTask(keys[i:i+10], self.priority)
+            self.tasks.append(t)
+            sync.submitTask(t)
 
 class SyncProjectTask(Task):
     _closed_statuses = ['MERGED', 'ABANDONED']
@@ -229,14 +278,16 @@ class SyncProjectTask(Task):
                 if batch and '_more_changes' in batch[-1]:
                     sortkey = '&N=%s' % (batch[-1]['_sortkey'],)
                     done = False
+        change_ids = [c['id'] for c in changes]
         with app.db.getSession() as session:
-            for c in changes:
-                # For now, just sync open changes or changes already
-                # in the db optionally we could sync all changes ever
-                change = session.getChangeByID(c['id'])
-                if change or (c['status'] not in self._closed_statuses):
-                    sync.submitTask(SyncChangeTask(c['id'], priority=self.priority))
-                    self.log.debug("Change %s update %s" % (c['id'], c['updated']))
+            # Winnow the list of IDs to only the ones in the local DB.
+            change_ids = session.getChangeIDs(change_ids)
+
+        for c in changes:
+            # For now, just sync open changes or changes already
+            # in the db optionally we could sync all changes ever
+            if c['id'] in change_ids or (c['status'] not in self._closed_statuses):
+                sync.submitTask(SyncChangeTask(c['id'], priority=self.priority))
         for key in self.project_keys:
             sync.submitTask(SetProjectUpdatedTask(key, now, priority=self.priority))
 
@@ -269,10 +320,9 @@ class SyncChangeByCommitTask(Task):
             query = 'commit:%s' % self.commit
         changes = sync.get('changes/?q=%s' % query)
         self.log.debug('Query: %s ' % (query,))
-        with app.db.getSession() as session:
-            for c in changes:
-                sync.submitTask(SyncChangeTask(c['id'], priority=self.priority))
-                self.log.debug("Sync change %s for its commit %s" % (c['id'], self.commit))
+        for c in changes:
+            sync.submitTask(SyncChangeTask(c['id'], priority=self.priority))
+            self.log.debug("Sync change %s for its commit %s" % (c['id'], self.commit))
 
 class SyncChangeByNumberTask(Task):
     def __init__(self, number, priority=NORMAL_PRIORITY):
@@ -288,12 +338,11 @@ class SyncChangeByNumberTask(Task):
             query = '%s' % self.number
         changes = sync.get('changes/?q=%s' % query)
         self.log.debug('Query: %s ' % (query,))
-        with app.db.getSession() as session:
-            for c in changes:
-                task = SyncChangeTask(c['id'], priority=self.priority)
-                self.tasks.append(task)
-                sync.submitTask(task)
-                self.log.debug("Sync change %s because it is number %s" % (c['id'], self.number))
+        for c in changes:
+            task = SyncChangeTask(c['id'], priority=self.priority)
+            self.tasks.append(task)
+            sync.submitTask(task)
+            self.log.debug("Sync change %s because it is number %s" % (c['id'], self.number))
 
 class SyncChangeTask(Task):
     def __init__(self, change_id, force_fetch=False, priority=NORMAL_PRIORITY):
@@ -305,6 +354,7 @@ class SyncChangeTask(Task):
         return '<SyncChangeTask %s>' % (self.change_id,)
 
     def run(self, sync):
+        start_time = time.time()
         app = sync.app
         remote_change = sync.get('changes/%s?o=DETAILED_LABELS&o=ALL_REVISIONS&o=ALL_COMMITS&o=MESSAGES&o=DETAILED_ACCOUNTS&o=CURRENT_ACTIONS' % self.change_id)
         # Perform subqueries this task will need outside of the db session
@@ -327,8 +377,19 @@ class SyncChangeTask(Task):
                                               remote_change['subject'], created,
                                               updated, remote_change['status'],
                                               topic=remote_change.get('topic'))
+                self.log.info("Created new change %s in local DB.", change.id)
+                result = ChangeAddedEvent(change)
+            else:
+                result = ChangeUpdatedEvent(change)
+            self.results.append(result)
             change.owner = account
-            change.status = remote_change['status']
+            if change.status != remote_change['status']:
+                change.status = remote_change['status']
+                result.status_changed = True
+            if remote_change.get('starred'):
+                change.starred = True
+            else:
+                change.starred = False
             change.subject = remote_change['subject']
             change.updated = dateutil.parser.parse(remote_change['updated'])
             change.topic = remote_change.get('topic')
@@ -356,6 +417,7 @@ class SyncChangeTask(Task):
                                                      remote_revision['commit']['message'], remote_commit,
                                                      remote_revision['commit']['parents'][0]['commit'],
                                                      auth, ref)
+                    self.log.info("Created new revision %s for change %s revision %s in local DB.", revision.key, self.change_id, remote_revision['_number'])
                     new_revision = True
                 revision.message = remote_revision['commit']['message']
                 # TODO: handle multiple parents
@@ -367,6 +429,7 @@ class SyncChangeTask(Task):
                     sync.submitTask(SyncChangeByCommitTask(revision.parent, self.priority))
                     self.log.debug("Change %s revision %s needs parent commit %s synced" %
                                    (change.id, remote_revision['_number'], revision.parent))
+                result.updateRelatedChanges(session, change)
                 remote_comments_data = remote_revision['_gertty_remote_comments_data']
                 for remote_file, remote_comments in remote_comments_data.items():
                     for remote_comment in remote_comments:
@@ -386,6 +449,7 @@ class SyncChangeTask(Task):
                                                              created,
                                                              remote_file, parent, remote_comment.get('line'),
                                                              remote_comment['message'])
+                            self.log.info("Created new comment %s for revision %s in local DB.", comment.key, revision.key)
                         else:
                             if comment.author != account:
                                 comment.author = account
@@ -403,10 +467,14 @@ class SyncChangeTask(Task):
                 message = session.getMessageByID(remote_message['id'])
                 if not message:
                     revision = session.getRevisionByNumber(change, remote_message.get('_revision_number', 1))
-                    # Normalize date -> created
-                    created = dateutil.parser.parse(remote_message['date'])
-                    message = revision.createMessage(remote_message['id'], account, created,
+                    if revision:
+                        # Normalize date -> created
+                        created = dateutil.parser.parse(remote_message['date'])
+                        message = revision.createMessage(remote_message['id'], account, created,
                                                      remote_message['message'])
+                        self.log.info("Created new review message %s for revision %s in local DB.", message.key, revision.key)
+                    else:
+                        self.log.info("Unable to create new review message for revision %s because it is not in local DB (draft?).", remote_message.get('_revision_number'))
                 else:
                     if message.author != account:
                         message.author = account
@@ -435,7 +503,10 @@ class SyncChangeTask(Task):
             local_labels = {}
             for approval in change.approvals:
                 key = '%s~%s' % (approval.category, approval.reviewer.id)
-                local_approvals[key] = approval
+                if key in local_approvals:
+                    session.delete(approval)
+                else:
+                    local_approvals[key] = approval
             local_approval_keys = set(local_approvals.keys())
             for label in change.labels:
                 key = '%s~%s~%s' % (label.category, label.value, label.description)
@@ -457,6 +528,7 @@ class SyncChangeTask(Task):
                 change.createApproval(account,
                                       remote_approval['category'],
                                       remote_approval['value'])
+                self.log.info("Created approval for change %s in local DB.", change.id)
 
             for key in remote_label_keys-local_label_keys:
                 remote_label = remote_label_entries[key]
@@ -499,7 +571,9 @@ class SyncChangeTask(Task):
             if not user_voted:
                 # Only consider changing the reviewed state if we don't have a vote
                 if new_revision or new_message:
-                    change.reviewed = False
+                    if change.reviewed:
+                        change.reviewed = False
+                        result.review_flag_changed = True
         for url, refs in fetches.items():
             self.log.debug("Fetching from %s with refs %s", url, refs)
             try:
@@ -511,6 +585,9 @@ class SyncChangeTask(Task):
                 for ref in refs:
                     self.log.debug("git fetch %s %s" % (url, ref))
                     repo.fetch(url, ref)
+        end_time = time.time()
+        total_time = end_time - start_time
+        self.log.info("Synced change %s in %0.5f seconds.", self.change_id, total_time)
 
 class CheckReposTask(Task):
     # on startup, check all projects
@@ -521,17 +598,17 @@ class CheckReposTask(Task):
 
     def run(self, sync):
         app = sync.app
-        to_check = []
         with app.db.getSession() as session:
-            for project in session.getProjects(subscribed=True):
-                try:
-                    repo = app.getRepo(project.name)
-                    if repo.newly_cloned or app.fetch_missing_refs:
-                        to_check.append(project.key)
-                except Exception:
-                    self.log.exception("Exception checking repo %s" % (project.name,))
-        for key in to_check:
-            sync.submitTask(CheckRevisionsTask(key, priority=LOW_PRIORITY))
+            projects = session.getProjects(subscribed=True)
+        for project in projects:
+            try:
+                repo = app.getRepo(project.name)
+                if repo.newly_cloned or app.fetch_missing_refs:
+                    sync.submitTask(CheckRevisionsTask(key,
+                                                       priority=LOW_PRIORITY))
+            except Exception:
+                self.log.exception("Exception checking repo %s" %
+                                   (project.name,))
 
 class CheckRevisionsTask(Task):
     def __init__(self, project_key, priority=NORMAL_PRIORITY):
@@ -601,6 +678,8 @@ class UploadReviewsTask(Task):
                 sync.submitTask(RebaseChangeTask(c.key, self.priority))
             for c in session.getPendingStatusChanges():
                 sync.submitTask(ChangeStatusTask(c.key, self.priority))
+            for c in session.getPendingStarred():
+                sync.submitTask(ChangeStarredTask(c.key, self.priority))
             for c in session.getPendingCherryPicks():
                 sync.submitTask(SendCherryPickTask(c.key, self.priority))
             for r in session.getPendingCommitMessages():
@@ -644,6 +723,27 @@ class RebaseChangeTask(Task):
             sync.post('changes/%s/rebase' % (change.id,), {})
             sync.submitTask(SyncChangeTask(change.id, priority=self.priority))
 
+class ChangeStarredTask(Task):
+    def __init__(self, change_key, priority=NORMAL_PRIORITY):
+        super(ChangeStarredTask, self).__init__(priority)
+        self.change_key = change_key
+
+    def __repr__(self):
+        return '<ChangeStarredTask %s>' % (self.change_key,)
+
+    def run(self, sync):
+        app = sync.app
+        with app.db.getSession() as session:
+            change = session.getChange(self.change_key)
+            if change.starred:
+                sync.put('accounts/self/starred.changes/%s' % (change.id,),
+                         data={})
+            else:
+                sync.delete('accounts/self/starred.changes/%s' % (change.id,),
+                            data={})
+            change.pending_starred = False
+            sync.submitTask(SyncChangeTask(change.id, priority=self.priority))
+
 class ChangeStatusTask(Task):
     def __init__(self, change_key, priority=NORMAL_PRIORITY):
         super(ChangeStatusTask, self).__init__(priority)
@@ -781,6 +881,10 @@ class Sync(object):
         self.app = app
         self.log = logging.getLogger('gertty.sync')
         self.queue = MultiQueue([HIGH_PRIORITY, NORMAL_PRIORITY, LOW_PRIORITY])
+        self.result_queue = Queue.Queue()
+        # Disable InsecureRequestWarning when certificate validation is disabled
+        if not self.app.config.verify_ssl:
+            requests.packages.urllib3.disable_warnings()
         self.session = requests.Session()
         if self.app.config.auth_type == 'basic':
             authclass = requests.auth.HTTPBasicAuth
@@ -792,7 +896,7 @@ class Sync(object):
         self.submitTask(CheckReposTask(HIGH_PRIORITY))
         self.submitTask(UploadReviewsTask(HIGH_PRIORITY))
         self.submitTask(SyncProjectListTask(HIGH_PRIORITY))
-        self.submitTask(SyncSubscribedProjectsTask(HIGH_PRIORITY))
+        self.submitTask(SyncSubscribedProjectsTask(NORMAL_PRIORITY))
         self.submitTask(SyncSubscribedProjectBranchesTask(LOW_PRIORITY))
         self.periodic_thread = threading.Thread(target=self.periodicSync)
         self.periodic_thread.daemon = True
@@ -838,6 +942,8 @@ class Sync(object):
             self.app.status.update(error=True, refresh=False)
         self.offline = False
         self.app.status.update(offline=False, refresh=False)
+        for r in task.results:
+            self.result_queue.put(r)
         os.write(pipe, 'refresh\n')
         return None
 
@@ -853,9 +959,15 @@ class Sync(object):
                          headers = {'Accept': 'application/json',
                                     'Accept-Encoding': 'gzip',
                                     'User-Agent': self.user_agent})
-        self.log.debug('Received: %s' % (r.text,))
-        ret = json.loads(r.text[4:])
-        return ret
+        if r.status_code == 200:
+            ret = json.loads(r.text[4:])
+            if len(ret):
+                self.log.debug('200 OK, Received: %s' % (ret,))
+            else:
+                self.log.debug('200 OK, No body.')
+            return ret
+        else:
+            self.log.warn('HTTP response: %d', r.status_code)
 
     def post(self, path, data):
         url = self.url(path)
@@ -887,6 +999,17 @@ class Sync(object):
                                         'User-Agent': self.user_agent})
         self.log.debug('Received: %s' % (r.text,))
 
+    def delete(self, path, data):
+        url = self.url(path)
+        self.log.debug('DELETE: %s' % (url,))
+        self.log.debug('data: %s' % (data,))
+        r = self.session.delete(url, data=json.dumps(data).encode('utf8'),
+                                verify=self.app.config.verify_ssl,
+                                auth=self.auth,
+                                headers = {'Content-Type': 'application/json;charset=UTF-8',
+                                           'User-Agent': self.user_agent})
+        self.log.debug('Received: %s' % (r.text,))
+
     def syncSubscribedProjects(self):
         task = SyncSubscribedProjectsTask(LOW_PRIORITY)
         self.submitTask(task)
